/*****************************************************
Project :  bat_tester 
Chip type               : ATtiny13A
AVR Core Clock frequency: 9.600000 MHz
*****************************************************/
#include <tiny13a.h>
#include <delay.h>

#define Led_R PORTB.2
#define Led_G PORTB.1
#define Led_B PORTB.0

#define U1 275 // R .   "275"=2.75V    ,  -  ,  -  
#define U2 300 // R 
#define U3 320 // G 
#define U4 350 // B 

#define U_K 98 //  ""...  ,    Ux ("275"=2.75V)   ADC  U=1.1V,   18k/4.7k    50 
                //     "" -    .  .
 
unsigned char out_counter=0, flicker=0, flicker_counter=0, Led=0;
unsigned int adc_avg=0;
#define ADC_VREF_TYPE 0x40

unsigned int read_adc(unsigned char adc_input)
{ ADMUX=adc_input | (ADC_VREF_TYPE & 0xff); //delay_us(10);
  ADCSRA|=0x40; while ((ADCSRA & 0x10)==0);
  ADCSRA|=0x10;
  return ADCW;
}

interrupt [TIM0_COMPA] void timer0_compa_isr(void)
{ TCNT0=0x00; //500Hz
  if (out_counter) {out_counter--;} else {out_counter=3;}
  if (flicker_counter) {flicker_counter--;} else {flicker_counter=125; flicker=!flicker;} // 2Hz   
  
  if (out_counter==0)       {Led_R=!(Led&0x01);Led_G=1;Led_B=1;}
  else if (out_counter==1)  {Led_R=1;Led_G=!(Led&0x02);Led_B=1;}
  else if (out_counter==2)  {Led_R=1;Led_G=1;Led_B=!(Led&0x04);}
  else                      
  { Led_R=1;Led_G=1;Led_B=1;
    
    delay_us(500); //          ,     .        
    adc_avg+=read_adc(2)-adc_avg/50; 
    if      (adc_avg<U1*U_K) {Led=0;}                 // off
    else if (adc_avg<U2*U_K) {Led=(flicker==1);}      // R . 
    else if (adc_avg<U3*U_K) {Led=0x01;}              // R              
    else if (adc_avg<U4*U_K) {Led=0x02;}              // G 
    else                     {Led=0x04;}              // B 
  } 
}

void main(void)
{ 
  unsigned char tmp;
#pragma optsize-
  CLKPR=0x80; CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

  PORTB=0x07; DDRB=0x07;
  TCCR0A=0x00;TCCR0B=0x04;TCNT0=0x00;OCR0A=0x4A;OCR0B=0x00;GIMSK=0x00;MCUCR=0x20;TIMSK0=0x04;
  ACSR=0x80;ADCSRB=0x00;DIDR0=0x00;DIDR0&=0x03;DIDR0|=0x10;ADMUX=ADC_VREF_TYPE & 0xff;ADCSRA=0x87;

  #asm("sei")
  tmp=50; while (tmp) {tmp--; adc_avg+=read_adc(2);}
  
  while (1)
  { #asm ("sleep"); }
}